#include <signal.h>
#include <stdlib.h>
#include <sys/socket.h>
#include <unistd.h>
#include <afblib/hostport.h>
#include <afblib/outbuf.h>
#include <afblib/outbuf_printf.h>
#include "mastermind-session.h"

#define DEFAULT_PORT 33031

int main(int argc, char** argv) {
   char* cmdname = *argv++; --argc;
   outbuf err = {2};
   if (argc != 1) {
      outbuf_printf(&err, "Usage: %s hostport\n",
	 cmdname);
      outbuf_flush(&err);
      exit(1);
   }
   char* hostport_string = *argv++; --argc;
   char* hangman_file = *argv++; --argc;

   hostport hp;
   if (!parse_hostport(hostport_string, &hp, DEFAULT_PORT)) {
      outbuf_printf(&err, "%s: bad hostport: %s\n",
	 cmdname, hostport_string);
      outbuf_flush(&err);
      exit(1);
   }

   /* run the service */
   int sfd = socket(hp.domain, SOCK_STREAM, hp.protocol);
   int optval = 1;
   if (sfd < 0 ||
        setsockopt(sfd, SOL_SOCKET, SO_REUSEADDR,
                   &optval, sizeof optval) < 0 ||
        bind(sfd, (struct sockaddr *) &hp.addr, hp.namelen) < 0 ||
        listen(sfd, SOMAXCONN) < 0) {
      outbuf_printf(&err, "%s: cannot bind to %s\n",
	 cmdname, hostport_string);
      outbuf_flush(&err);
      exit(1);
   }

   /* our children shall not become zombies */
   struct sigaction action = {
      .sa_handler = SIG_IGN,
      .sa_flags = SA_NOCLDWAIT,
   };
   sigaction(SIGCHLD, &action, 0);

   int fd;
   while ((fd = accept(sfd, 0, 0)) >= 0) {
      pid_t child = fork();
      if (child < 0) {
	 close(fd); close(sfd); break;
      }
      if (child == 0) {
	 close(sfd);
	 mastermind_session(fd);
         exit(0);
      }
      close(fd);
   }
}
